c **** subroutine: dposv.for **** Version: 13 Jul 2002 *****************
c  dposv computes the solution to a real system of linear equations:
c        a*x=b,
c    where a is an n-by-n symmetric positive definite matrix and x and
c    b are n-by-nrhs matrices.
c    The Cholesky decomposition is used to factor a as
c        a=u^T*u,  if uplo='u', or
c        a=l*l^T,  if uplo='l',
c    where u is an upper triangular matrix and l is a lower triangular
c    matrix.  The factored form of a is then used to solve the system of
c    equations a*x=b.
c  Arguments:
c  uplo    (input) character*1
c          ='u':  upper triangle of a is stored;
c          ='l':  lower triangle of a is stored.
c  n       (input) integer
c          The number of linear equations, i.e., the order of the
c          matrix a.  n >= 0.
c  nrhs    (input) integer
c          The number of right hand sides, i.e., the number of columns
c          of the matrix b.  nrhs >= 0.
c  a       (input/output) real*8 array, dimension (lda,n)
c          On entry, the symmetric matrix a.  If uplo='u', the leading
c          n-by-n upper triangular part of a contains the upper
c          triangular part of the matrix a, and the strictly lower
c          triangular part of a is not referenced.  If uplo='l', the
c          leading n-by-n lower triangular part of a contains the lower
c          triangular part of the matrix a, and the strictly upper
c          triangular part of a is not referenced.
c          On exit, if info=0, the factor u or l from the Cholesky
c          factorization a=u^T*u or a=l*l^T.
c  lda     (input) integer
c          The leading dimension of the array a.  lda >= max(1,n).
c  b       (input/output) real*8 array, dimension (ldb,nrhs)
c          On entry, the n-by-nrhs right hand side matrix b.
c          On exit, if info=0, the n-by-nrhs solution matrix x.
c  ldb     (input) integer
c          The leading dimension of the array b.  ldb >= max(1,n).
c  info    (output) integer
c          =0:  successful exit
c          < 0:  if info=-i, the i-th argument had an illegal value
c          > 0:  if info=i, the leading minor of order i of a is not
c                positive definite, so the factorization could not be
c                completed, and the solution has not been computed.
c  LAPACK driver routine (Version 3.0)
c     Univ. of Tennessee, Univ. of California Berkeley, NAG ltd.,
c     Courant Institute, Argonne National Lab, and Rice University
c     March 31, 1993
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine dposv(uplo,n,nrhs,a,lda,b,ldb,info)
      implicit none
c
c  Arguments:
c
      character uplo*1
      integer info,lda,ldb,n,nrhs
      real*8 a(lda,*),b(ldb,*)
c
c  External functions:
c
      logical lsame
c
c  Test the input parameters.
      info=0
      if(.not.lsame(uplo,'u').and..not.lsame(uplo,'l'))then
        info=-1
      elseif(n.lt.0)then
        info=-2
      elseif(nrhs.lt.0)then
        info=-3
      elseif(lda.lt.max(1,n))then
        info=-5
      elseif(ldb.lt.max(1,n))then
        info=-7
      endif
      if(info.ne.0)then
        call xerbla('dposv ',-info)
        return
      endif
c
c  Compute the Cholesky factorization a=u'*u or a=l*l'.
      call dpotrf(uplo,n,a,lda,info)
c
c  Solve the system a*x=b, overwriting b with x.
      if(info.eq.0)then
        call dpotrs(uplo,n,nrhs,a,lda,b,ldb,info)
      endif
c
      return
      end
c
c **** subroutine: dpotrf.for **** Version: 17 Jul 2002 ****************
c  dpotrf computes the Cholesky factorization of a real symmetric
c    positive definite matrix a.
c    The factorization has the form
c        a=u^T*u, if uplo='u',or
c        a=l*l^T, if uplo='l',
c    where u is an upper triangular matrix and l is lower triangular.
c    This is the block version of the algorithm, calling level 3 blas.
c  Arguments:
c  uplo    (input) character*1
c          ='u':  upper triangle of a is stored;
c          ='l':  lower triangle of a is stored.
c  n       (input) integer
c          The order of the matrix a.  n >= 0.
c  a       (input/output) real*8 array, dimension (lda,n)
c          On entry, the symmetric matrix a.  If uplo='u', the leading
c          n-by-n upper triangular part of a contains the upper
c          triangular part of the matrix a, and the strictly lower
c          triangular part of a is not referenced.  If uplo='l', the
c          leading n-by-n lower triangular part of a contains the lower
c          triangular part of the matrix a, and the strictly upper
c          triangular part of a is not referenced.
c          On exit, if info=0, the factor u or l from the Cholesky
c          factorization a=u^T*u or a=l*l^T.
c  lda     (input) integer
c          The leading dimension of the array a.  lda >= max(1,n).
c  info    (output) integer
c          =0:  successful exit
c          <0:  if info=-i, the i-th argument had an illegal value
c          >0:  if info=i, the leading minor of order i is not
c                positive definite, and the factorization could not be
c                completed.
c  LAPACK routine (version 3.0)
c     Univ. of Tennessee, Univ. of California Berkeley, NAG ltd.,
c     Courant Institute, Argonne National Lab, and Rice University
c     March 31, 1993
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine dpotrf(uplo,n,a,lda,info)
      implicit none
c
c  Arguments:
c
      character uplo*1
      integer info,lda,n
      real*8 a(lda,*)
c
c  Local variables:
c
      integer j,jb,nb
      logical upper
      real*8 one
      parameter (one=1d0)
c
c  External functions:
c
      integer ilaenv
      logical lsame
c
c  Test the input parameters.
      info=0
      upper=lsame(uplo,'u')
      if(.not.upper.and..not.lsame(uplo,'l'))then
        info=-1
      elseif(n.lt.0)then
        info=-2
      elseif(lda.lt.max(1,n))then
        info=-4
      endif
      if(info.ne.0)then
        call xerbla('dpotrf',-info)
        return
      endif
c
c  Quick return if possible
      if(n.eq.0)return
c
c     determine the block size for this environment.
      nb=ilaenv(1,'dpotrf',uplo,n,-1,-1,-1)
      if(nb.le.1.or.nb.ge.n)then
c
c  Use unblocked code.
        call dpotf2(uplo,n,a,lda,info)
      else
c
c  Use blocked code.
        if(upper)then
c
c  Compute the Cholesky factorization a=u'*u.
          do j=1,n,nb
c
c  Update and factorize the current diagonal block and test
c    for non-positive-definiteness.
            jb=min(nb,n-j+1)
            call dsyrk('upper','transpose',jb,j-1,-one,
     &          a(1,j),lda,one,a(j,j),lda)
            call dpotf2('upper',jb,a(j,j),lda,info)
            if(info.ne.0)goto30
            if(j+jb.le.n)then
c
c  Compute the current block row.
              call dgemm('transpose','no transpose',jb,n-j-jb+1,
     &            j-1,-one,a(1,j),lda,a(1,j+jb),
     &            lda,one,a(j,j+jb),lda)
              call dtrsm('left','upper','transpose','non-unit',
     &            jb,n-j-jb+1,one,a(j,j),lda,a(j,j+jb),lda)
            endif
          enddo
        else
c
c  Compute the Cholesky factorization a=l*l'.
          do j=1,n,nb
c
c  Update and factorize the current diagonal block and test
c    for non-positive-definiteness.
            jb=min(nb,n-j+1)
            call dsyrk('lower','no transpose',jb,j-1,-one,
     &          a(j,1),lda,one,a(j,j),lda)
            call dpotf2('lower',jb,a(j,j),lda,info)
            if(info.ne.0)goto30
            if(j+jb.le.n)then
c
c  Compute the current block column.
              call dgemm('no transpose','transpose',n-j-jb+1,jb,
     &            j-1,-one,a(j+jb,1),lda,a(j,1),lda,one,a(j+jb,j),lda)
              call dtrsm('right','lower','transpose','non-unit',
     &            n-j-jb+1,jb,one,a(j,j),lda,a(j+jb,j),lda)
            endif
          enddo
        endif
      endif
      goto40
c
30    info=info+j-1
c
40    continue
c
      return
      end
c
c **** subroutine: dpotrs.for **** Version: 17 Jul 2002 ****************
c  dpotrs solves a system of linear equations a*x=b with a symmetric
c    positive definite matrix a using the Cholesky factorization
c    a=u^T*u or a=l*l^T computed by dpotrf.
c  Arguments:
c  uplo    (input) character*1
c          ='u':  upper triangle of a is stored;
c          ='l':  lower triangle of a is stored.
c  n       (input) integer
c          The order of the matrix a.  n >= 0.
c  nrhs    (input) integer
c          The number of right hand sides, i.e., the number of columns
c          of the matrix b.  nrhs >= 0.
c  a       (input) real*8 array, dimension (lda,n)
c          The triangular factor u or l from the Cholesky factorization
c          a=u^T*u or a=l*l^T, as computed by dpotrf.
c  lda     (input) integer
c          The leading dimension of the array a.  lda >= max(1,n).
c  b       (input/output) real*8 array, dimension (ldb,nrhs)
c          On entry, the right hand side matrix b.
c          On exit, the solution matrix x.
c  ldb     (input) integer
c          The leading dimension of the array b.  ldb >= max(1,n).
c  info    (output) integer
c          =0:  successful exit
c          <0:  if info=-i, the i-th argument had an illegal value
c  LAPACK routine (version 3.0)
c     Univ. of Tennessee, Univ. of California Berkeley, NAG ltd.,
c     Courant Institute, Argonne National Lab, and Rice University
c     March 31, 1993
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine dpotrs(uplo,n,nrhs,a,lda,b,ldb,info)
      implicit none
c
c  Arguments:
c
      character uplo*1
      integer info,lda,ldb,n,nrhs
      real*8 a(lda,*),b(ldb,*)
c
c  Local variables:
c
      logical upper
      real*8 one
      parameter (one=1d0)
c
c  External functions:
c
      logical lsame
c
c  Test the input parameters.
      info=0
      upper=lsame(uplo,'u')
      if(.not.upper.and..not.lsame(uplo,'l'))then
        info=-1
      elseif(n.lt.0)then
        info=-2
      elseif(nrhs.lt.0)then
        info=-3
      elseif(lda.lt.max(1,n))then
        info=-5
      elseif(ldb.lt.max(1,n))then
        info=-7
      endif
      if(info.ne.0)then
        call xerbla('dpotrs',-info)
        return
      endif
c
c  Quick return if possible
      if(n.eq.0.or.nrhs.eq.0)return
c
      if(upper)then
c
c  Solve a*x=b where a=u'*u:
c  Solve u'*x=b, overwriting b with x.
        call dtrsm('left','upper','transpose','non-unit',n,nrhs,
     &      one,a,lda,b,ldb)
c
c  Solve u*x=b, overwriting b with x.
        call dtrsm('left','upper','no transpose','non-unit',n,
     &      nrhs,one,a,lda,b,ldb)
      else
c
c  Solve a*x=b where a=l*l':
c  Solve l*x=b, overwriting b with x.
        call dtrsm('left','lower','no transpose','non-unit',n,
     &      nrhs,one,a,lda,b,ldb)
c
c  Solve l'*x=b,overwriting b with x.
        call dtrsm('left','lower','transpose','non-unit',n,nrhs,
     &      one,a,lda,b,ldb)
      endif
c
      return
      end
c
c **** subroutine: dpotf2.for **** Version: 17 Jul 2002 ****************
c  dpotf2 computes the Cholesky factorization of a real symmetric
c    positive definite matrix a.
c    The factorization has the form
c        a=u'*u, if uplo='u', or
c        a=l*l', if uplo='l',
c    where u is an upper triangular matrix and l is lower triangular.
c    This is the unblocked version of the algorithm, calling level 2 blas.
c  Arguments:
c  uplo    (input) character*1
c          Specifies whether the upper or lower triangular part of the
c          symmetric matrix a is stored.
c          ='u':  upper triangular
c          ='l':  lower triangular
c  n       (input) integer
c          The order of the matrix a.  n >= 0.
c  a       (input/output) real*8 array, dimension (lda,n)
c          On entry, the symmetric matrix a.  If uplo='u', the leading
c          n by n upper triangular part of a contains the upper
c          triangular part of the matrix a, and the strictly lower
c          triangular part of a is not referenced.  If uplo='l', the
c          leading n by n lower triangular part of a contains the lower
c          triangular part of the matrix a, and the strictly upper
c          triangular part of a is not referenced.
c          On exit, if info=0, the factor u or l from the Cholesky
c          factorization a=u'*u or a=l*l'.
c  lda     (input) integer
c          The leading dimension of the array a.  lda >= max(1,n).
c  info    (output) integer
c          =0: successful exit
c          <0: if info=-k, the k-th argument had an illegal value
c          >0: if info=k, the leading minor of order k is not
c               positive definite, and the factorization could not be
c               completed.
c  LAPACK routine (version 3.0)
c     Univ. of Tennessee, Univ. of California Berkeley, NAG ltd.,
c     Courant Institute, Argonne National Lab, and Rice University
c     February 29, 1992
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine dpotf2(uplo,n,a,lda,info)
      implicit none
c
c  Arguments:
c
      character uplo*1
      integer info,lda,n
      real*8 a(lda,*)
c
c  Local variables:
c
      integer j
      logical upper
      real*8 ajj,one,zero
      parameter (one=1d0,zero=0d0)
c
c  External functions:
c
      logical lsame
      real*8 ddot
c
c  Test the input parameters.
      info=0
      upper=lsame(uplo,'u')
      if(.not.upper.and..not.lsame(uplo,'l'))then
        info=-1
      elseif(n.lt.0)then
        info=-2
      elseif(lda.lt.max(1,n))then
        info=-4
      endif
      if(info.ne.0)then
        call xerbla('dpotf2',-info)
        return
      endif
c
c  Quick return if possible
      if(n.eq.0)return
c
      if(upper)then
c
c  Compute the Cholesky factorization a=u'*u.
        do j=1,n
c
c  Compute u(j,j) and test for non-positive-definiteness.
          ajj=a(j,j)-ddot(j-1,a(1,j),1,a(1,j),1)
          if(ajj.le.zero)then
            a(j,j)=ajj
            goto30
          endif
          ajj=dsqrt(ajj)
          a(j,j)=ajj
c
c  Compute elements j+1:n of row j.
          if(j.lt.n)then
            call dgemv('transpose',j-1,n-j,-one,a(1,j+1),
     &          lda,a(1,j),1,one,a(j,j+1),lda)
            call dscal(n-j,one/ajj,a(j,j+1),lda)
          endif
        enddo
      else
c
c  Compute the Cholesky factorization a=l*l'.
        do j=1,n
c
c  Compute l(j,j) and test for non-positive-definiteness.
          ajj=a(j,j)-ddot(j-1,a(j,1),lda,a(j,1),lda)
          if(ajj.le.zero)then
            a(j,j)=ajj
            goto30
          endif
          ajj=dsqrt(ajj)
          a(j,j)=ajj
c
c  Compute elements j+1:n of column j.
          if(j.lt.n)then
            call dgemv('no transpose',n-j,j-1,-one,a(j+1,1),
     &          lda,a(j,1),lda,one,a(j+1,j),1)
            call dscal(n-j,one/ajj,a(j+1,j),1)
          endif
        enddo
      endif
      goto40
c
30    info=j
c
40    continue
c
      return
      end
c
c **** subroutine: dsyrk.for **** Version: 18 Jul 2002 *****************
c  dsyrk performs one of the symmetric rank k operations
c        c=alpha*a*a'+beta*c,
c    or
c        c=alpha*a'*a+beta*c,
c    where alpha and beta are scalars, c is an n-by-n symmetric matrix
c    and a is an n-by-k matrix in the first case and a k-by-n matrix
c    in the second case.
c  Arguments:
c  uplo    (input) character*1.
c          ='u':  only the upper triangular part of c is referenced.
c          ='l':  only the lower triangular part of c is referenced.
c  trans   (input) character*1.
c           ='n':  c=alpha*a*a'+beta*c.
c           ='t':  c=alpha*a'*a+beta*c.
c           ='c':  c=alpha*a'*a+beta*c.
c  n        (input) integer.
c           The number of linear equations, i.e., the order of the 
c           matrix c.  n >= 0.
c  k        (input) integer.
c           trans='n':  specifies the number of columns of the matrix a.
c           trans='t' or ='c':  specifies the number of rows of the matrix a.  
c           k >= 0.
c  alpha    (input) real*8.
c           Specifies the scalar alpha.
c  a        (input) real*8 
c           Array of dimension (lda,ka), where ka is
c           k when trans='n' and is n otherwise.
c           trans='n':  the leading n by k part of a must contain the matrix.
c           Otherwise the leading k by n part must contain the matrix.
c  lda      (input) integer.
c           The first dimension of a as declared in the calling (sub)program.
c           trans='n': lda >= max(1,n).
c           Otherwise lda >= max(1,k).
c  beta     (input) real*8.
c           beta specifies the scalar beta.
c  c        (input/output) real*8, dimension (ldc,n).
c           uplo='u':  the leading n by n
c           upper triangular part of c must contain the upper
c           triangular part of the symmetric matrix and the strictly
c           lower triangular part of c is not referenced. On exit, the
c           upper triangular part of c is overwritten by the
c           upper triangular part of the updated matrix.
c           uplo='l':  the leading n by n
c           lower triangular part of c must contain the lower
c           triangular part of the symmetric matrix and the strictly
c           upper triangular part of c is not referenced.  On exit, the
c           lower triangular part of the array c is overwritten by the
c           lower triangular part of the updated matrix.
c  ldc      (input) integer.
c           Specifies the first dimension of c as declared
c           in the calling (sub)program.  ldc >= max(1,n).
c  Level 3 blas routine.
c     Written on 8-February-1989.
c     Jack Dongarra, Argonne National Laboratory.
c     Iain Duff, Aere Harwell.
c     Jeremy du Croz, Numerical Algorithms Group ltd.
c     Sven Hammarling, Numerical Algorithms Group ltd.
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine dsyrk(uplo,trans,n,k,alpha,a,lda,beta,c,ldc)
      implicit none
c
c  Arguments:
c
      character uplo*1,trans*1
      integer n,k,lda,ldc
      real*8 alpha,beta
      real*8 a(lda,*),c(ldc,*)
c
c  External functions:
c
      logical lsame
c
c  Local variables:
c
      integer i,info,j,l,nrowa
      logical upper
      real*8 temp,one,zero
      parameter (one=1d0,zero=0d0)
c
c  Test the input parameters.
      if(lsame(trans,'n'))then
        nrowa=n
      else
        nrowa=k
      endif
      upper=lsame(uplo,'u')
c
      info=0
      if(.not.upper.and..not.lsame(uplo,'l'))then
        info=1
      elseif(.not.lsame(trans,'n').and..not.lsame(trans,'t').and.
     &    .not.lsame(trans,'c'))then
        info=2
      elseif(n.lt.0)then
        info=3
      elseif(k.lt.0)then
        info=4
      elseif(lda.lt.max(1,nrowa))then
        info=7
      elseif(ldc.lt.max(1,n))then
        info=10
      endif
      if(info.ne.0)then
        call xerbla('dsyrk ',info)
        return
      endif
c
c  Quick return if possible.
      if((n.eq.0).or.((alpha.eq.zero.or.k.eq.0).and.beta.eq.one))return
c
c  And when alpha.eq.zero.
      if(alpha.eq.zero)then
        if(upper)then
          if(beta.eq.zero)then
            do j=1,n
              do i=1,j
                c(i,j)=zero
              enddo
            enddo
          else
            do j=1,n
              do i=1,j
                c(i,j)=beta*c(i,j)
              enddo
            enddo
          endif
        else
          if(beta.eq.zero)then
            do j=1,n
              do i=j,n
                c(i,j)=zero
              enddo
            enddo
          else
            do j=1,n
              do i=j,n
                c(i,j)=beta*c(i,j)
              enddo
            enddo
          endif
        endif
        return
      endif
c
c  Start the operations.
      if(lsame(trans,'n'))then
c
c  Form c=alpha*a*a'+beta*c.
        if(upper)then
          do j=1,n
            if(beta.eq.zero)then
              do i=1,j
                c(i,j)=zero
              enddo
            elseif(beta.ne.one)then
              do i=1,j
                c(i,j)=beta*c(i,j)
              enddo
            endif
            do l=1,k
              if(a(j,l).ne.zero)then
                temp=alpha*a(j,l)
                do i=1,j
                  c(i,j)=c(i,j)+temp*a(i,l)
                enddo
              endif
            enddo
          enddo
        else
          do j=1,n
            if(beta.eq.zero)then
              do i=j,n
                c(i,j)=zero
              enddo
            elseif(beta.ne.one)then
              do i=j,n
                c(i,j)=beta*c(i,j)
              enddo
            endif
            do l=1,k
              if(a(j,l).ne.zero)then
                temp     =alpha*a(j,l)
                do i=j,n
                  c(i,j)=c(i,j)+temp*a(i,l)
                enddo
              endif
            enddo
          enddo
        endif
      else
c
c  Form c=alpha*a'*a+beta*c.
        if(upper)then
          do j=1,n
            do i=1,j
              temp=zero
              do l=1,k
                temp=temp+a(l,i)*a(l,j)
              enddo
              if(beta.eq.zero)then
                c(i,j)=alpha*temp
              else
                c(i,j)=alpha*temp+beta*c(i,j)
              endif
            enddo
          enddo
        else
          do j=1,n
            do i=j,n
              temp=zero
              do l=1,k
                temp=temp+a(l,i)*a(l,j)
              enddo
              if(beta.eq.zero)then
                c(i,j)=alpha*temp
              else
                c(i,j)=alpha*temp+beta*c(i,j)
              endif
            enddo
          enddo
        endif
      endif
c
      return
      end
c
c **** subroutine: dtrsm.for **** Version: 21 Jul 2002 *****************
c  dtrsm solves one of the matrix equations
c        op(a)*x=alpha*b, or x*op(a)=alpha*b,
c  where alpha is a scalar, x and b are m-by-n matrices, a is a unit, or
c  non-unit, upper or lower triangular matrix and op(a) is one of
c        op(a)=a or op(a)=a'.
c  The matrix x is overwritten on b.
c  Arguments:
c  side     (input) character*1.
c           side specifies whether op(a) appears on the left
c           or right of x as follows:
c              side='l': op(a)*x=alpha*b.
c              side='r': x*op(a)=alpha*b.
c  uplo     (input) character*1.
c           uplo specifies whether the matrix a is an upper or
c           lower triangular matrix as follows:
c              uplo='u': a is an upper triangular matrix.
c              uplo='l': a is a lower triangular matrix.
c  transa   (input) character*1.
c           transa specifies the form of op(a) to be used in
c           the matrix multiplication as follows:
c              transa='n': op(a)=a.
c              transa='t': op(a)=a'.
c              transa='c': op(a)=a'.
c  diag     (input) character*1.
c           diag specifies whether or not a is unit triangular
c           as follows:
c              diag='u': a is assumed to be unit triangular.
c              diag='n': a is not assumed to be unit triangular.
c  m        (input) integer.
c           m specifies the number of rows of b. m must be at
c           least zero.
c  n        (input) integer.
c           n specifies the number of columns of b.  n must be
c           at least zero.
c  alpha    (input) real*8.
c           alpha specifies the scalar alpha. When alpha is
c           zero then a is not referenced and b need not be set before
c           entry.
c  a        (input) real*8 array of dimension (lda,k), where k is m
c           when side='l' and is n when side='r'.
c           Before entry with uplo='u', the leading k-by-k
c           upper triangular part of the array a must contain the upper
c           triangular matrix and the strictly lower triangular part of
c           a is not referenced.
c           Before entry with uplo='l', the leading k-by-k
c           lower triangular part of the array a must contain the lower
c           triangular matrix and the strictly upper triangular part of
c           a is not referenced.
c           Note that when diag='u', the diagonal elements of
c           a are not referenced either, but are assumed to be unity.
c  lda      (input) integer.
c           lda specifies the first dimension of a as declared
c           in the calling (sub) program.  When side='l' then
c           lda must be at least max(1,m), when side='r'
c           then lda must be at least max(1,n).
c  b        (input/output) real*8 array of dimension (ldb,n).
c           Before entry, the leading m-by-n part of the array b must
c           contain the right-hand side matrix b, and on exit is
c           overwritten by the solution matrix x.
c  ldb      (input) integer.
c           ldb specifies the first dimension of b as declared
c           in the calling (sub) program.  ldb must be at least
c           max(1,m).
c  Level 3 blas routine.
c     Jack Dongarra, Argonne National Laboratory.
c     Iain Duff, Aere Harwell.
c     Jeremy du Croz, Numerical Algorithms Group ltd.
c     Sven Hammarling, Numerical Algorithms Group ltd.
c     Written on 8-February-1989.
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine dtrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb)
      implicit none
c
c  Arguments:
c
      character side*1,uplo*1,transa*1,diag*1
      integer m,n,lda,ldb
      real*8 alpha,a(lda,*),b(ldb,*)
c
c  External functions:
c
      logical lsame
c
c  Local variables:
c
      logical lside,nounit,upper
      integer i,info,j,k,nrowa
      real*8 temp,one,zero
      parameter (one=1d0,zero=0d0)
c
c  Test the input parameters.
      lside=lsame(side,'l')
      if(lside)then
        nrowa=m
      else
        nrowa=n
      endif
      nounit=lsame(diag,'n')
      upper=lsame(uplo,'u')
c
      info=0
      if(.not.lside.and..not.lsame(side,'r'))then
        info=1
      elseif(.not.upper.and..not.lsame(uplo,'l'))then
        info=2
      elseif(.not.lsame(transa,'n').and..not.lsame(transa,'t').and.
     &    .not.lsame(transa,'c'))then
        info=3
      elseif(.not.lsame(diag,'u').and..not.lsame(diag,'n'))then
        info=4
      elseif(m.lt.0)then
        info=5
      elseif(n.lt.0)then
        info=6
      elseif(lda.lt.max(1,nrowa))then
        info=9
      elseif(ldb.lt.max(1,m))then
        info=11
      endif
      if(info.ne.0)then
        call xerbla('dtrsm ',info)
        return
      endif
c
c  Quick return if possible.
      if(n.eq.0)return
c
c  And when alpha.eq.zero.
      if(alpha.eq.zero)then
        do j=1,n
          do i=1,m
            b(i,j)=zero
          enddo
        enddo
        return
      endif
c
c  Start the operations.
      if(lside)then
        if(lsame(transa,'n'))then
c
c  Form b=alpha*inv(a)*b.
          if(upper)then
            do j=1,n
              if(alpha.ne.one)then
                do i=1,m
                  b(i,j)=alpha*b(i,j)
                enddo
              endif
              do k=m,1,-1
                if(b(k,j).ne.zero)then
                  if(nounit)b(k,j)=b(k,j)/a(k,k)
                  do i=1,k-1
                    b(i,j)=b(i,j)-b(k,j)*a(i,k)
                  enddo
                endif
              enddo
            enddo
          else
            do j=1,n
              if(alpha.ne.one)then
                do i=1,m
                  b(i,j)=alpha*b(i,j)
                enddo
              endif
              do k=1,m
                if(b(k,j).ne.zero)then
                  if(nounit)b(k,j)=b(k,j)/a(k,k)
                  do i=k+1,m
                    b(i,j)=b(i,j)-b(k,j)*a(i,k)
                  enddo
                endif
              enddo
            enddo
          endif
        else
c
c  Form b=alpha*inv(a')*b.
          if(upper)then
            do j=1,n
              do i=1,m
                temp=alpha*b(i,j)
                do k=1,i-1
                  temp=temp-a(k,i)*b(k,j)
                enddo
                if(nounit)temp=temp/a(i,i)
                b(i,j)=temp
              enddo
            enddo
          else
            do j=1,n
              do i=m,1,-1
                temp=alpha*b(i,j)
                do k=i+1,m
                  temp=temp-a(k,i)*b(k,j)
                enddo
                if(nounit)temp=temp/a(i,i)
                b(i,j)=temp
              enddo
            enddo
          endif
        endif
      else
        if(lsame(transa,'n'))then
c
c  Form b=alpha*b*inv(a).
          if(upper)then
            do j=1,n
              if(alpha.ne.one)then
                do i=1,m
                  b(i,j)=alpha*b(i,j)
                enddo
              endif
              do k=1,j-1
                if(a(k,j).ne.zero)then
                  do i=1,m
                    b(i,j)=b(i,j)-a(k,j)*b(i,k)
                  enddo
                endif
              enddo
              if(nounit)then
                temp=one/a(j,j)
                do i=1,m
                  b(i,j)=temp*b(i,j)
                enddo
              endif
            enddo
          else
            do j=n,1,-1
              if(alpha.ne.one)then
                do i=1,m
                  b(i,j)=alpha*b(i,j)
                enddo
              endif
              do k=j+1,n
                if(a(k,j).ne.zero)then
                  do i=1,m
                    b(i,j)=b(i,j)-a(k,j)*b(i,k)
                  enddo
                endif
              enddo
              if(nounit)then
                temp=one/a(j,j)
                do i=1,m
                  b(i,j)=temp*b(i,j)
                enddo
              endif
            enddo
          endif
        else
c
c  Form b=alpha*b*inv(a').
          if(upper)then
            do k=n,1,-1
              if(nounit)then
                temp=one/a(k,k)
                do i=1,m
                  b(i,k)=temp*b(i,k)
                enddo
              endif
              do j=1,k-1
                if(a(j,k).ne.zero)then
                  temp=a(j,k)
                  do i=1,m
                    b(i,j)=b(i,j)-temp*b(i,k)
                  enddo
                endif
              enddo
              if(alpha.ne.one)then
                do i=1,m
                  b(i,k)=alpha*b(i,k)
                enddo
              endif
            enddo
          else
            do k=1,n
              if(nounit)then
                temp=one/a(k,k)
                do i=1,m
                  b(i,k)=temp*b(i,k)
                enddo
              endif
              do j=k+1,n
                if(a(j,k).ne.zero)then
                  temp=a(j,k)
                  do i=1,m
                    b(i,j)=b(i,j)-temp*b(i,k)
                  enddo
                endif
              enddo
              if(alpha.ne.one)then
                do i=1,m
                  b(i,k)=alpha*b(i,k)
                enddo
              endif
            enddo
          endif
        endif
      endif
c
      return
      end
c
